/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2008-2012 SonarSource
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.core.component;

import com.google.common.collect.Maps;
import org.sonar.api.BatchComponent;
import org.sonar.api.batch.SonarIndex;
import org.sonar.api.component.Component;
import org.sonar.api.component.Perspective;
import org.sonar.api.component.ResourcePerspectives;
import org.sonar.api.resources.Resource;

import javax.annotation.CheckForNull;

import java.util.Map;

public class ScanPerspectives implements ResourcePerspectives, BatchComponent {
  private final ScanGraph graph;
  private final Map<Class<?>, PerspectiveBuilder<?>> builders = Maps.newHashMap();
  private final SonarIndex resourceIndex;

  public ScanPerspectives(ScanGraph graph, PerspectiveBuilder[] builders, SonarIndex resourceIndex) {
    this.graph = graph;
    this.resourceIndex = resourceIndex;
    for (PerspectiveBuilder builder : builders) {
      // TODO check duplications
      this.builders.put(builder.getPerspectiveClass(), builder);
    }
  }

  @CheckForNull
  public <P extends Perspective> P as(Class<P> perspectiveClass, Component component) {
    if (component.key() == null) {
      return null;
    }

    ComponentVertex vertex;
    if (component instanceof ComponentVertex) {
      vertex = (ComponentVertex) component;
    } else {
      vertex = graph.getComponent(component.key());
    }

    if (vertex != null) {
      PerspectiveBuilder<P> builder = builderFor(perspectiveClass);
      P perspective = builder.load(vertex);
      if (perspective == null) {
        perspective = builder.create(vertex);
      }
      return perspective;
    }
    return null;
  }

  public <P extends Perspective> P as(Class<P> perspectiveClass, Resource resource) {
    Resource indexedResource = resourceIndex.getResource(resource);
    if (indexedResource != null) {
      return as(perspectiveClass, new ResourceComponent(indexedResource));
    }
    return null;
  }

  private <T extends Perspective> PerspectiveBuilder<T> builderFor(Class<T> clazz) {
    PerspectiveBuilder<T> builder = (PerspectiveBuilder<T>) builders.get(clazz);
    if (builder == null) {
      throw new PerspectiveNotFoundException("Perspective class is not registered: " + clazz);
    }
    return builder;
  }
}
